﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Threading.Tasks;
using DirectShowDemo.Contracts;

namespace DirectShowDemo
{
    public class CameraFrameSource : IFrameSource
    {
        public event Action<IFrameSource, byte[], Size> NewFrame;

        public bool IsCapturing { get; private set; }

        private Camera _camera;
        public Camera Camera
        {
            get { return _camera; }

            internal set
            {
                if (_camera != value)
                {
                    bool restart = IsCapturing;
                    if (IsCapturing)
                    {
                        StopFrameCapture();
                    }

                    _camera = value;

                    if (restart)
                    {
                        StartFrameCapture();
                    }
                }
            }
        }

        public CameraFrameSource(Camera camera)
        {
            if (camera == null) throw new ArgumentNullException("camera");

            this.Camera = camera;
        }

        public void StartFrameCapture()
        {
            if (!IsCapturing && this.Camera != null)
            {
                this.Camera.OnImageCaptured += OnImageCaptured;
                this.Camera.StartCapture();
                IsCapturing = true;
            }
        }

        public void StopFrameCapture()
        {
            if (IsCapturing)
            {
                this.Camera.StopCapture();
                this.Camera.OnImageCaptured -= OnImageCaptured;
                this.IsCapturing = false;
            }
        }
        public void ReleaseEvents()
        {
            StopFrameCapture();
            NewFrame = null;
        }
        private void OnImageCaptured(object sender, CameraEventArgs e)
        {
            if (IsCapturing && this.NewFrame != null)
            {
                this.NewFrame(this, e.Image, e.ImageSize);
            }
        }
    }
}
