﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using DirectShowDemo.Contracts;

namespace DirectShowDemo
{
    public partial class Demo : Form
    {
        private CameraFrameSource _frameSource;
        private static string _lockimg = "";
        private int _xoffsett = 0;
        private int _yoffsett = 0;
        private int _xstep = 1;
        private int _ystep = 1;

        public Demo()
        {
            InitializeComponent();
        }

        private void thrashOldCamera()
        {
            // Trash the old camera
            if (_frameSource != null)
            {
                _frameSource.ReleaseEvents();
                _frameSource.Camera.Dispose();
                setFrameSource(null);
            }
        }

        private void setFrameSource(CameraFrameSource cameraFrameSource)
        {
            if (_frameSource == cameraFrameSource)
                return;

            _frameSource = cameraFrameSource;
        }

        private void OnReadImage(IFrameSource frameSource, byte[] data, Size sz)
        {
            lock (_lockimg)
            {
                BitmapData bf = null;
                Bitmap bmp = null;
                try
                {
                    bmp = new Bitmap(sz.Width, sz.Height);
                    bf = bmp.LockBits(new Rectangle(0, 0, sz.Width, sz.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppRgb);
                    unsafe
                    {
                        byte* bmpdata = (byte*)bf.Scan0;
                        int pos = 3 * ((_xoffsett * (sz.Width - 1)) + (sz.Width * (_yoffsett * (sz.Height - 1))));
                        int xstep = 3 * _xstep;
                        int offlin = _ystep * 3 * sz.Width;
                        int imgpos = 0;
                        for (int iy = 0; iy < sz.Height; iy++)
                        {
                            int posi = pos;
                            for (int ix = 0; ix < sz.Width; ix++)
                            {
                                bmpdata[imgpos++] = data[pos];
                                bmpdata[imgpos++] = data[pos + 1];
                                bmpdata[imgpos++] = data[pos + 2];
                                bmpdata[imgpos++] = 255;
                                pos += xstep;
                            }
                            pos = posi + offlin;
                        }
                    }
                    bmp.UnlockBits(bf);
                    bf = null;
                    pbCamera.BeginInvoke((Action)delegate() { pbCamera.Image = bmp; });
                }
                catch
                {
                    if (bf != null)
                    {
                        bmp.UnlockBits(bf);
                        bf = null;
                    }
                }
            }
        }

        private void startVideo()
        {
            try
            {
                Camera c = (Camera)cbCameras.SelectedItem;
                c.CaptureWidth = Convert.ToInt32(txtWidth.Text);
                c.CaptureHeight = Convert.ToInt32(txtHeight.Text);
                setFrameSource(new CameraFrameSource(c));
                _frameSource.NewFrame += OnReadImage;
                try
                {
                    if (c is VideoFile)
                    {
                        _frameSource.Camera.ShowPropertiesDialog();
                    }
                    else
                    {
                        Thread t = new Thread(new ThreadStart(_frameSource.Camera.ShowPropertiesDialog));
                        t.Start();
                    }
                }
                catch
                {
                }
                _frameSource.StartFrameCapture();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void Demo_Load(object sender, EventArgs e)
        {
            if (!DesignMode)
            {
                // Refresh the list of available cameras
                cbCameras.Items.Clear();
                foreach (Camera cam in CameraService.AvailableCameras)
                    cbCameras.Items.Add(cam);

                if (cbCameras.Items.Count > 0)
                    cbCameras.SelectedIndex = 0;
            }
        }

        private void bPlay_Click(object sender, EventArgs e)
        {
            if (_frameSource != null && _frameSource.Camera == cbCameras.SelectedItem)
                return;
            try
            {
                bStop.Enabled = true;
                bPlay.Enabled = false;
                pbCamera.Image = null;
                thrashOldCamera();
                startVideo();
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null)
                {
                    MessageBox.Show(ex.Message + "\n" + ex.InnerException.Message);
                }
                else
                {
                    MessageBox.Show(ex.Message);
                }
                bPlay.Enabled = true;
                bStop.Enabled = false;
            }
        }

        private void bFlipH_Click(object sender, EventArgs e)
        {
            bFlipH.Checked = !bFlipH.Checked;
            if (bFlipH.Checked)
            {
                _xoffsett = 1;
                _xstep = -1;
            }
            else
            {
                _xoffsett = 0;
                _xstep = 1;
            }
        }

        private void bFlipV_Click(object sender, EventArgs e)
        {
            bFlipV.Checked = !bFlipV.Checked;
            if (bFlipV.Checked)
            {
                _yoffsett = 1;
                _ystep = -1;
            }
            else
            {
                _yoffsett = 0;
                _ystep = 1;
            }
        }

        private void bStop_Click(object sender, EventArgs e)
        {
            bPlay.Enabled = true;
            bStop.Enabled = false;
            thrashOldCamera();
        }

        private void cbCameras_SelectedIndexChanged(object sender, EventArgs e)
        {
            Camera cam = null;
            if (cbCameras.SelectedItem != null)
            {
                cam = cbCameras.SelectedItem as Camera;
            }
            if (cam != null)
            {
                txtWidth.Text = cam.CaptureWidth.ToString();
                txtHeight.Text = cam.CaptureHeight.ToString();
            }
            else
            {
                txtWidth.Text = "";
                txtHeight.Text = "";
            }
        }

        private void Demo_FormClosing(object sender, FormClosingEventArgs e)
        {
            thrashOldCamera();
        }
    }
}
