// WebCamLib.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include <dshow.h>
#include "qedit.h"
#include <strsafe.h>


#ifdef _MANAGED
#pragma managed(push, off)
#endif

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
    return TRUE;
}

////// Private variables
#define MAX_CAMERAS		10

struct CameraInfo
{
	BSTR bstrName;
	IMoniker* pMoniker;
};
struct CameraResolution
{
	int nWidth;
	int nHeight;
};

IGraphBuilder* g_pGraphBuilder = NULL;
IMediaControl* g_pMediaControl = NULL;
ICaptureGraphBuilder2* g_pCaptureGraphBuilder = NULL;
IBaseFilter* g_pIBaseFilterCam = NULL;
IBaseFilter* g_pIBaseFilterSampleGrabber = NULL;
IBaseFilter* g_pIBaseFilterNullRenderer = NULL;
IBaseFilter* g_pIBaseFilterVideoFile = NULL;
CameraInfo g_aCameraInfo[MAX_CAMERAS] = {0};

typedef void (__stdcall *PFN_CaptureCallback)(DWORD dwSize, BYTE* pbData);
PFN_CaptureCallback g_pfnCaptureCallback = NULL;

///// Forward declarations
HRESULT ConfigureSampleGrabber(IBaseFilter *pIBaseFilter);
HRESULT ConfigureFileSampleGrabber(IBaseFilter *pIBaseFilter);
IPin* GetPin(IBaseFilter* pFilter, PIN_DIRECTION pinDir, int index);
void CleanupCameraInfo();
extern "C" __declspec(dllexport) DWORD APIENTRY Cleanup();
extern "C" __declspec(dllexport) DWORD APIENTRY StopCamera();

////// Exported functions
extern "C"
{

__declspec(dllexport) DWORD APIENTRY RefreshCameraList(int *nCount)
{
	IEnumMoniker* pclassEnum = NULL;
	ICreateDevEnum* pdevEnum = NULL;

	if (NULL == nCount)
	{
		return E_INVALIDARG;
	}

	*nCount = 0;

	CleanupCameraInfo();

	HRESULT hr = CoCreateInstance(CLSID_SystemDeviceEnum, 
			NULL, 
			CLSCTX_INPROC, 
			IID_ICreateDevEnum, 
			(LPVOID*)&pdevEnum);

	if (SUCCEEDED(hr))
	{
        hr = pdevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, &pclassEnum, 0);
	}

	if (pdevEnum != NULL)
	{
		pdevEnum->Release();
		pdevEnum = NULL;
	}

	if (pclassEnum != NULL)
	{ 
	    IMoniker* apIMoniker[1];
		ULONG ulCount = 0;

        while (SUCCEEDED(hr) && (*nCount) < MAX_CAMERAS && pclassEnum->Next(1, apIMoniker, &ulCount) == S_OK)
        {
			g_aCameraInfo[*nCount].pMoniker = apIMoniker[0];

			IPropertyBag *pPropBag;
			hr = apIMoniker[0]->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag);
	        if (SUCCEEDED(hr))
	        {
	            // To retrieve the filter's friendly name, do the following:
	            VARIANT varName;
	            VariantInit(&varName);
	            hr = pPropBag->Read(L"FriendlyName", &varName, 0);
	            if (SUCCEEDED(hr) && varName.vt == VT_BSTR)
	            {
					g_aCameraInfo[*nCount].bstrName = SysAllocString(varName.bstrVal);
	            }
	            VariantClear(&varName);
	            
	            pPropBag->Release();
	        }

			(*nCount)++;
        }

		pclassEnum->Release();
	}

	return hr;
}

__declspec(dllexport) DWORD APIENTRY GetCameraDetails(int index, IUnknown **ppUnk, BSTR* pbstrName)
{
	if (index >= MAX_CAMERAS || NULL == g_aCameraInfo[index].pMoniker || NULL == pbstrName)
	{
		return E_INVALIDARG;
	}

	g_aCameraInfo[index].pMoniker->AddRef();
	*ppUnk = g_aCameraInfo[index].pMoniker;
	*pbstrName = SysAllocString(g_aCameraInfo[index].bstrName);

	return S_OK;
}

__declspec(dllexport) DWORD APIENTRY DisplayCameraPropertiesDialog(IUnknown *pUnk, HWND hwnd)
{
	if (NULL == pUnk)
	{
		return E_INVALIDARG;
	}

	HRESULT hr = S_OK;
	IMoniker *pMoniker = NULL;
	IBaseFilter *pFilter = NULL;
	ISpecifyPropertyPages *pProp = NULL;

	// Grab the moniker interface
	hr = pUnk->QueryInterface(IID_IMoniker, (LPVOID*)&pMoniker);

	// Create a filter graph for the moniker
	if (SUCCEEDED(hr))
	{ 
		hr = pMoniker->BindToObject(NULL, NULL, IID_IBaseFilter, (LPVOID*)&pFilter);
	}

	// See if it implements a property page
	if (SUCCEEDED(hr))
	{
		hr = pFilter->QueryInterface(IID_ISpecifyPropertyPages, (LPVOID*)&pProp);
	}

	// Show the property page
	if (SUCCEEDED(hr))
	{
		FILTER_INFO filterinfo;
		hr = pFilter->QueryFilterInfo(&filterinfo);

		IUnknown *pFilterUnk = NULL;
		if (SUCCEEDED(hr))
		{
			hr = pFilter->QueryInterface(IID_IUnknown, (LPVOID*)&pFilterUnk);
		}

		if (SUCCEEDED(hr))
		{
			CAUUID caGUID;
			pProp->GetPages(&caGUID);

			OleCreatePropertyFrame(
		        hwnd,                   // Parent window
		        0, 0,                   // Reserved
		        filterinfo.achName,     // Caption for the dialog box
		        1,                      // Number of objects (just the filter)
		        &pFilterUnk,            // Array of object pointers. 
		        caGUID.cElems,          // Number of property pages
		        caGUID.pElems,          // Array of property page CLSIDs
		        0,                      // Locale identifier
		        0, NULL                 // Reserved
				);
		}

		if (pFilterUnk != NULL)
		{
			pFilterUnk->Release();
			pFilterUnk = NULL;
		}
	}

	if (pProp != NULL)
	{
		pProp->Release();
		pProp = NULL;
	}

	if (pMoniker != NULL)
	{
		pMoniker->Release();
		pMoniker = NULL;
	}

	if (pFilter != NULL)
	{
		pFilter->Release();
		pFilter = NULL;
	}

	return hr;
}

__declspec(dllexport) DWORD APIENTRY StartCamera(IUnknown *pUnk, PFN_CaptureCallback pfnCallback, int minwidth, int minheight, int* pnWidth, int* pnHeight, int* pnStride)
{
	if (NULL == pUnk || NULL == pfnCallback || NULL == pnWidth || NULL == pnHeight)
	{
		return E_INVALIDARG;
	}

	if (g_pGraphBuilder != NULL)
	{
		return E_UNEXPECTED;
	}

	g_pfnCaptureCallback = pfnCallback;
	
	IMoniker *pMoniker = NULL;
	HRESULT hr = S_OK;
	HRESULT res = S_OK;

	// Grab the moniker interface
	hr = pUnk->QueryInterface(IID_IMoniker, (LPVOID*)&pMoniker);

	// Build all the necessary interfaces to start the capture
	if (SUCCEEDED(hr))
	{
		hr = CoCreateInstance(CLSID_FilterGraph, 
			NULL, 
			CLSCTX_INPROC, 
			IID_IGraphBuilder, 
			(LPVOID*)&g_pGraphBuilder);
	}
	else
	{
		res = 1;
	}
	if (SUCCEEDED(hr))
	{
		hr = g_pGraphBuilder->QueryInterface(IID_IMediaControl, (LPVOID*)&g_pMediaControl);
	}
	else
	{
		if (res == S_OK) res = 2;
	}
	if (SUCCEEDED(hr))
	{
		hr = CoCreateInstance(CLSID_CaptureGraphBuilder2, 
			NULL, 
			CLSCTX_INPROC, 
			IID_ICaptureGraphBuilder2, 
			(LPVOID*)&g_pCaptureGraphBuilder);
	}
	else
	{
		if (res == S_OK) res = 3;
	}
	// Setup the filter graph
	if (SUCCEEDED(hr))
	{
		 hr = g_pCaptureGraphBuilder->SetFiltergraph(g_pGraphBuilder);
	}
	else
	{
		if (res == S_OK) res = 4;
	}
	// Build the camera from the moniker
	if (SUCCEEDED(hr))
	{ 
		hr = pMoniker->BindToObject(NULL, NULL, IID_IBaseFilter, (LPVOID*)&g_pIBaseFilterCam);
	}
	else
	{
		if (res == S_OK) res = 5;
	}
	// Add the camera to the filter graph
	if (SUCCEEDED(hr))
	{
		hr = g_pGraphBuilder->AddFilter(g_pIBaseFilterCam, L"VideoCam");
	}
	else
	{
		if (res == S_OK) res = 6;
	}
	// Create a SampleGrabber
	if (SUCCEEDED(hr))
	{
		hr = CoCreateInstance(CLSID_SampleGrabber, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void**)&g_pIBaseFilterSampleGrabber);
	}
	else
	{
		if (res == S_OK) res = 7;
	}
	// Configure the Sample Grabber
	if (SUCCEEDED(hr))
	{
		hr = ConfigureSampleGrabber(g_pIBaseFilterSampleGrabber);
	}
	else
	{
		if (res == S_OK) res = 8;
	}
	// Add Sample Grabber to the filter graph
	if (SUCCEEDED(hr))
	{
		hr = g_pGraphBuilder->AddFilter(g_pIBaseFilterSampleGrabber, L"SampleGrabber");
	}
	else
	{
		if (res == S_OK) res = 9;
	}
	// Create the NullRender
	if (SUCCEEDED(hr))
	{
		hr = CoCreateInstance(CLSID_NullRenderer, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void**)&g_pIBaseFilterNullRenderer);
	}
	else
	{
		if (res == S_OK) res = 10;
	}
	// Add the Null Render to the filter graph
	if (SUCCEEDED(hr))
	{
		hr = g_pGraphBuilder->AddFilter(g_pIBaseFilterNullRenderer, L"NullRenderer");
	}
	else
	{
		if (res == S_OK) res = 11;
	}
	// Configure the render stream
	if (SUCCEEDED(hr))
	{
		IAMStreamConfig *pConfig = NULL;
		hr = g_pCaptureGraphBuilder->FindInterface(
			&PIN_CATEGORY_CAPTURE, // Capture pin.
			0,    // Any media type.
			g_pIBaseFilterCam, // Pointer to the capture filter.
			IID_IAMStreamConfig, (void**)&pConfig);
		if (SUCCEEDED(hr))
		{
			int iCount = 0, iSize = 0;
			hr = pConfig->GetNumberOfCapabilities(&iCount, &iSize);

			// Check the size to make sure we pass in the correct structure.
			if (iSize == sizeof(VIDEO_STREAM_CONFIG_CAPS))
			{
				// Use the video capabilities structure.
				for (int iFormat = 0; iFormat < iCount; iFormat++)
				{					
					VIDEO_STREAM_CONFIG_CAPS scc;
					AM_MEDIA_TYPE *pmtConfig;
					hr = pConfig->GetStreamCaps(iFormat, &pmtConfig, (BYTE*)&scc);
					if (SUCCEEDED(hr))
					{
						/* Examine the format, and possibly use it. */
						VIDEOINFOHEADER *pVih = (VIDEOINFOHEADER*)pmtConfig->pbFormat;
						if ((pmtConfig->majortype == MEDIATYPE_Video) &&
							(pmtConfig->formattype == FORMAT_VideoInfo) &&
							(pmtConfig->cbFormat >= sizeof(VIDEOINFOHEADER)) &&
							(pmtConfig->pbFormat != NULL) &&
							((pVih->bmiHeader.biWidth >= minwidth) &&
							(pVih->bmiHeader.biHeight >= minheight)) &&
							(pVih->bmiHeader.biHeight <= 1080))
						{
							// pVih contains the detailed format information.
							pConfig->SetFormat(pmtConfig);
							break;
						}

						// Delete the media type when you are done.
						// FreeMediaType(pmtConfig);
					}
				}
			}

			hr = g_pCaptureGraphBuilder->RenderStream(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, g_pIBaseFilterCam, g_pIBaseFilterSampleGrabber, g_pIBaseFilterNullRenderer);
		}
		else
		{
			if (res == S_OK) res = 13;
		}
	}
	else
	{
		if (res == S_OK) res = 12;
	}

	// Grab the capture width and height
	if (SUCCEEDED(hr))
	{
		ISampleGrabber* pGrabber = NULL;
		hr = g_pIBaseFilterSampleGrabber->QueryInterface(IID_ISampleGrabber, (LPVOID*)&pGrabber);
		if (SUCCEEDED(hr))
		{
			AM_MEDIA_TYPE mt;
			hr = pGrabber->GetConnectedMediaType(&mt);
			if (SUCCEEDED(hr))
			{
				VIDEOINFOHEADER *pVih;
				if ((mt.formattype == FORMAT_VideoInfo) && 
				    (mt.cbFormat >= sizeof(VIDEOINFOHEADER)) &&
				    (mt.pbFormat != NULL) ) 
				{
				    pVih = (VIDEOINFOHEADER*)mt.pbFormat;
					*pnWidth = pVih->bmiHeader.biWidth;
					*pnHeight = pVih->bmiHeader.biHeight;
					*pnStride = pVih->bmiHeader.biWidth * (pVih->bmiHeader.biBitCount / 8);
				}
				else
				{
					hr = E_FAIL;  // Wrong format
				}

				//FreeMediaType(mt);  TODO: Fix this as its leaking
			}
			else
			{
				if (res == S_OK) res = 17;
			}
		}

		if (pGrabber != NULL)
		{
			pGrabber->Release();
			pGrabber = NULL;
		}
	}
	else
	{
		if (res == S_OK) res = 16;
	}
	// Start the capture
	if (SUCCEEDED(hr))
	{
		hr = g_pMediaControl->Run();
	}
	else
	{
		if (res == S_OK) res = 18;
	}
	// If init fails then ensure that you cleanup
	if (FAILED(hr))
	{
		StopCamera();
	}
	else
	{
		hr = S_OK;  // Make sure we return S_OK for success
	}

	// Cleanup
	if (pMoniker != NULL)
	{
		pMoniker->Release();
		pMoniker = NULL;
	}

	return res;
}

__declspec(dllexport) DWORD APIENTRY StartVideoFile(LPCWSTR wFileName, PFN_CaptureCallback pfnCallback, int* pnWidth, int* pnHeight, int* pnStride)
{	
	if (NULL == wFileName || NULL == pfnCallback || NULL == pnWidth || NULL == pnHeight)
	{
		return E_INVALIDARG;
	}

	if (g_pGraphBuilder != NULL)
	{
		return E_UNEXPECTED;
	}

	g_pfnCaptureCallback = pfnCallback;

	HRESULT hr = S_OK;
	HRESULT res = S_OK;

	// Build all the necessary interfaces to start the capture
	hr = CoCreateInstance(CLSID_FilterGraph,
		NULL,
		CLSCTX_INPROC,
		IID_IGraphBuilder,
		(LPVOID*)&g_pGraphBuilder);

	if (SUCCEEDED(hr))
	{
		hr = g_pGraphBuilder->QueryInterface(IID_IMediaControl, (LPVOID*)&g_pMediaControl);
	}
	else 
	{
		res = 1;
	}


	if (SUCCEEDED(hr))
	{
		hr = CoCreateInstance(CLSID_CaptureGraphBuilder2,
			NULL,
			CLSCTX_INPROC,
			IID_ICaptureGraphBuilder2,
			(LPVOID*)&g_pCaptureGraphBuilder);
	}
	else
	{
		if (res == S_OK) res = 2;
	}
	// Setup the filter graph
	if (SUCCEEDED(hr))
	{
		hr = g_pCaptureGraphBuilder->SetFiltergraph(g_pGraphBuilder);
	}
	else
	{
		if (res == S_OK) res = 3;
	}
	// Add the file to the filter graph
	if (SUCCEEDED(hr))
	{
		hr = g_pGraphBuilder->AddSourceFilter(wFileName, L"VideoFile", &g_pIBaseFilterVideoFile);
	}
	else
	{
		if (res == S_OK) res = 4;
	}

	// Create a SampleGrabber
	if (SUCCEEDED(hr))
	{
		hr = CoCreateInstance(CLSID_SampleGrabber, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void**)&g_pIBaseFilterSampleGrabber);
	}
	else
	{
		if (res == S_OK) res = 9;
	}
	// Configure the Sample Grabber
	if (SUCCEEDED(hr))
	{
		hr = ConfigureFileSampleGrabber(g_pIBaseFilterSampleGrabber);
	}
	else
	{
		if (res == S_OK) res = 10;
	}
	// Add Sample Grabber to the filter graph
	if (SUCCEEDED(hr))
	{
		hr = g_pGraphBuilder->AddFilter(g_pIBaseFilterSampleGrabber, L"SampleGrabber");
	}
	else
	{
		if (res == S_OK) res = 11;
	}
	// Create the NullRender
	if (SUCCEEDED(hr))
	{
		hr = CoCreateInstance(CLSID_NullRenderer, NULL, CLSCTX_INPROC_SERVER, IID_IBaseFilter, (void**)&g_pIBaseFilterNullRenderer);
	}
	else
	{
		if (res == S_OK) res = 12;
	}
	// Add the Null Render to the filter graph
	if (SUCCEEDED(hr))
	{
		hr = g_pGraphBuilder->AddFilter(g_pIBaseFilterNullRenderer, L"NullRenderer");
	}
	else
	{
		if (res == S_OK) res = 13;
	}
	// Configure the render stream	
	if (SUCCEEDED(hr))
	{
		hr = E_UNEXPECTED;
		IPin* pOutFile = GetPin(g_pIBaseFilterVideoFile, PINDIR_OUTPUT, 1);
		if (pOutFile != NULL) 
		{
			IPin* pInGrabber = GetPin(g_pIBaseFilterSampleGrabber, PINDIR_INPUT, 1);
			if (pInGrabber != NULL)
			{
				hr = g_pGraphBuilder->Connect(pOutFile, pInGrabber);
				if (SUCCEEDED(hr)) 
				{
					hr = E_UNEXPECTED;
					IPin* pOutGrabber = GetPin(g_pIBaseFilterSampleGrabber, PINDIR_OUTPUT, 1);
					if (pOutGrabber != NULL) 
					{
						IPin* pInRender = GetPin(g_pIBaseFilterNullRenderer, PINDIR_INPUT, 1);
						if (pInRender != NULL) 
						{
							hr = g_pGraphBuilder->Connect(pOutGrabber, pInRender);
						}
					}
				}
				else 
				{
					if (res == S_OK) res = 100;
				}
			}
		}
	}
	else
	{
		if (res == S_OK) res = 14;
	}
	// Grab the capture width and height
	if (SUCCEEDED(hr))
	{
		ISampleGrabber* pGrabber = NULL;
		hr = g_pIBaseFilterSampleGrabber->QueryInterface(IID_ISampleGrabber, (LPVOID*)&pGrabber);
		if (SUCCEEDED(hr))
		{
			AM_MEDIA_TYPE mt;
			hr = pGrabber->GetConnectedMediaType(&mt);
			if (SUCCEEDED(hr))
			{
				VIDEOINFOHEADER *pVih;
				if ((mt.formattype == FORMAT_VideoInfo) &&
					(mt.cbFormat >= sizeof(VIDEOINFOHEADER)) &&
					(mt.pbFormat != NULL))
				{
					pVih = (VIDEOINFOHEADER*)mt.pbFormat;
					*pnWidth = pVih->bmiHeader.biWidth;
					*pnHeight = pVih->bmiHeader.biHeight;
					*pnStride = (pVih->bmiHeader.biWidth * (pVih->bmiHeader.biBitCount / 8)) & ~3;
				}
				else
				{
					hr = E_FAIL;  // Wrong format
				}

				//FreeMediaType(mt);  TODO: Fix this as its leaking
			}
			else
			{
				if (res == S_OK) res = 17;
			}
		}
		else
		{
			if (res == S_OK) res = 16;
		}
		if (pGrabber != NULL)
		{
			pGrabber->Release();
			pGrabber = NULL;
		}
	}
	else
	{
		if (res == S_OK) res = 15;
	}
	// Start the capture
	if (SUCCEEDED(hr))
	{
		hr = g_pMediaControl->Run();
	}

	// If init fails then ensure that you cleanup
	if (FAILED(hr))
	{
		StopCamera();
	}
	else
	{
		hr = S_OK;  // Make sure we return S_OK for success
	}

	return res;
}

__declspec(dllexport) DWORD APIENTRY StopCamera()
{
	HRESULT hr = S_OK;

	if (g_pMediaControl != NULL)
	{
		g_pMediaControl->Stop();
		g_pMediaControl->Release();
		g_pMediaControl = NULL;
	}

	g_pfnCaptureCallback = NULL;

	if (g_pIBaseFilterNullRenderer != NULL)
	{
		g_pIBaseFilterNullRenderer->Release();
		g_pIBaseFilterNullRenderer = NULL;
	}

	if (g_pIBaseFilterSampleGrabber != NULL)
	{
		g_pIBaseFilterSampleGrabber->Release();
		g_pIBaseFilterSampleGrabber = NULL;
	}

	if (g_pIBaseFilterVideoFile != NULL) 
	{
		g_pIBaseFilterVideoFile->Release();
		g_pIBaseFilterVideoFile = NULL;
	}

	if (g_pIBaseFilterCam != NULL)
	{
		g_pIBaseFilterCam->Release();
		g_pIBaseFilterCam = NULL;
	}

	if (g_pGraphBuilder != NULL)
	{
		g_pGraphBuilder->Release();
		g_pGraphBuilder = NULL;
	}

	if (g_pCaptureGraphBuilder != NULL)
	{
		g_pCaptureGraphBuilder->Release();
		g_pCaptureGraphBuilder = NULL;
	}

	return hr;
}

__declspec(dllexport) DWORD APIENTRY Initialize()
{
	HRESULT hr = S_OK;

	// Nothing to really do yet

	return hr;
}

__declspec(dllexport) DWORD APIENTRY Cleanup()
{
	StopCamera();	
	CleanupCameraInfo();

	return S_OK;
}

}

void CleanupCameraInfo()
{
	for (int n = 0; n < MAX_CAMERAS; n++)
	{
		SysFreeString(g_aCameraInfo[n].bstrName);
        g_aCameraInfo[n].bstrName = NULL;

		if (g_aCameraInfo[n].pMoniker != NULL)
		{
			g_aCameraInfo[n].pMoniker->Release();
			g_aCameraInfo[n].pMoniker = NULL;
		}
	}
}

IPin* GetPin(IBaseFilter* pFilter, PIN_DIRECTION pinDir, int index)
{
	BOOL bFound = FALSE;
	IEnumPins* pEnum;
	IPin* pPin;
	int pindex = 1;

	HRESULT hr = pFilter->EnumPins(&pEnum);
	if (FAILED(hr)) 
	{
		return NULL;
	}
	while (pEnum->Next(1, &pPin, 0) == S_OK) 
	{
		PIN_DIRECTION pinDirThis;
		pPin->QueryDirection(&pinDirThis);
		if (bFound = (pinDir == pinDirThis)) 
		{
			if (pindex == index) 
			{
				break;
			}
			else 
			{
				pindex++;
			}
		}
		pPin->Release();
	}
	pEnum->Release();
	return (bFound ? pPin : NULL);
}

// SampleGrabber callback interface
class MySampleGrabberCB : public ISampleGrabberCB
{
public:

	MySampleGrabberCB()
	{
		m_nRefCount = 0;
	}

	 virtual HRESULT STDMETHODCALLTYPE SampleCB( 
            double SampleTime,
            IMediaSample *pSample)
	 {
		 return E_FAIL;
	 }
        
     virtual HRESULT STDMETHODCALLTYPE BufferCB( 
            double SampleTime,
            BYTE *pBuffer,
            long BufferLen)
	 {
		if (g_pfnCaptureCallback != NULL)
		{
			g_pfnCaptureCallback(BufferLen, pBuffer);
		}

		return S_OK;
	 }

	virtual HRESULT STDMETHODCALLTYPE QueryInterface( 
            REFIID riid,
            void **ppvObject)
	 {
		 return E_FAIL;  // Not a very accurate implementation
	 }
        
	virtual ULONG STDMETHODCALLTYPE AddRef()
	{
		return ++m_nRefCount;
	}
        
	virtual ULONG STDMETHODCALLTYPE Release()
	{
		int n = --m_nRefCount;
		if (n <= 0)
		{
			delete this;
		}
		return n;
	}

private:
	int m_nRefCount;
};

HRESULT ConfigureSampleGrabber(IBaseFilter *pIBaseFilter)
{
	HRESULT hr = S_OK;

	ISampleGrabber *pGrabber = NULL;

	hr = pIBaseFilter->QueryInterface(IID_ISampleGrabber, (void**)&pGrabber);
	if (SUCCEEDED(hr))
	{
		AM_MEDIA_TYPE mt;
		ZeroMemory(&mt, sizeof(AM_MEDIA_TYPE));
		mt.majortype = MEDIATYPE_Video;
		mt.subtype = MEDIASUBTYPE_RGB24;
		mt.formattype = FORMAT_VideoInfo;
		hr = pGrabber->SetMediaType(&mt);
	}

	if (SUCCEEDED(hr))
	{
		hr = pGrabber->SetCallback(new MySampleGrabberCB(), 1);
	}

	if (pGrabber != NULL)
	{
		pGrabber->Release();
		pGrabber = NULL;
	}

	return hr;
}

HRESULT ConfigureFileSampleGrabber(IBaseFilter *pIBaseFilter)
{
	HRESULT hr = S_OK;

	ISampleGrabber *pGrabber = NULL;

	hr = pIBaseFilter->QueryInterface(IID_ISampleGrabber, (void**)&pGrabber);
	if (SUCCEEDED(hr))
	{
		AM_MEDIA_TYPE mt;
		ZeroMemory(&mt, sizeof(AM_MEDIA_TYPE));
		mt.majortype = MEDIATYPE_Video;
		mt.subtype = MEDIASUBTYPE_RGB24;
		mt.formattype = FORMAT_VideoInfo;
		hr = pGrabber->SetMediaType(&mt);
	}

	if (SUCCEEDED(hr))
	{
		hr = pGrabber->SetCallback(new MySampleGrabberCB(), 1);
	}

	if (SUCCEEDED(hr))
	{
		hr = pGrabber->SetOneShot(FALSE);
	}

	if (pGrabber != NULL)
	{
		pGrabber->Release();
		pGrabber = NULL;
	}

	return hr;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif

